package client;

import service.*;

import java.io.*;

import java.rmi.*; 

import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.ServiceDiscoveryListener;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lease.LeaseRenewalManager;

// Change start
import java.rmi.server.ExportException;
import net.jini.export.*; 
import net.jini.jrmp.JrmpExporter;
import net.jini.core.event.*;
// Change end

import net.jini.lookup.LookupCache;

// Change start
public class ClientListener implements  ServiceDiscoveryListener,
					 RemoteEventListener {
    // Change end

    private LookupCache cache = null;

    // Change start
    private Remote proxy;
    // Change end

    public static void main(String[] args) {
	ClientListener cl = new ClientListener();

	// Change start
	// This is a hack to keep the server alive.
	Object keepAlive = new Object();
	synchronized(keepAlive) {
	    try {
		// Wait for a "notify" from another thread
		// that will never be sent.
		// So we stay alive for ever
		keepAlive.wait();
	    } catch(java.lang.InterruptedException e) {
		// do nothing
	    }
	}    
	// Change end
    }

    public ClientListener() {
	System.setSecurityManager(new RMISecurityManager());

        discoverOtherServices();

	// When this registers as an event listener it gives an
	// object to the service to call notify() on it
	// The object must be a proxy for this
	Exporter exporter = new JrmpExporter();
	try {
	    proxy = exporter.export(this);
	} catch(ExportException e) {
	    System.exit(1);
	}
    }

    private void discoverOtherServices() {
	// Build a cache of all discovered echo services and monitor changes
	ServiceDiscoveryManager serviceMgr = null;
	Class [] classes = new Class[] {Echo.class};
	ServiceTemplate template = new ServiceTemplate(null, classes, 
							   null);	
	try {
            LookupDiscoveryManager mgr =
                new LookupDiscoveryManager(LookupDiscovery.ALL_GROUPS,
                                           null,  // unicast locators
                                           null); // DiscoveryListener
            serviceMgr = new ServiceDiscoveryManager(mgr, 
						    new LeaseRenewalManager());
        } catch(Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
	
        try {
            cache = serviceMgr.createLookupCache(template, 
                                                null,  // no filter
                                                this); // listener
        } catch(Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    public void serviceAdded(ServiceDiscoveryEvent evt) {
	// evt.getPreEventServiceItem() == null
	ServiceItem postItem = evt.getPostEventServiceItem();
	System.out.println("Service appeared: " +
			   postItem.service.getClass().toString());
	// Change start
	Echo echo = (Echo) postItem.service;
	try {
	    echo.addEventListener((RemoteEventListener) proxy);
	} catch(RemoteException e) { 
	    e.printStackTrace(); 
	} 
	// Change end
    }

    public void serviceChanged(ServiceDiscoveryEvent evt) {
	ServiceItem preItem = evt.getPostEventServiceItem();
	ServiceItem postItem = evt.getPreEventServiceItem() ;
	System.out.println("Service changed: " +
			   postItem.service.getClass().toString());
    }
    public void serviceRemoved(ServiceDiscoveryEvent evt) {
	// evt.getPostEventServiceItem() == null
	ServiceItem preItem = evt.getPreEventServiceItem();
	System.out.println("Service disappeared: " +
			   preItem.service.getClass().toString());
    }

    // Change start
    public void notify(RemoteEvent e) {
	System.out.println("Echo service has been called ");
	String text = null;
	try {
	    Echo echoSrc = (Echo) e.getSource();
	    text = echoSrc.getChangedText();
	    System.out.println("Text changed to " + text);
	} catch(RemoteException re) {
	    re.printStackTrace();
	}
    }
    // Change end
}

